/**
 * The contents of this file are subject to the terms
 * of the Common Development and Distribution License
 * (the License). You may not use this file except in
 * compliance with the License.
 * 
 * You can obtain a copy of the license at
 * https://woodstock.dev.java.net/public/CDDLv1.0.html.
 * See the License for the specific language governing
 * permissions and limitations under the License.
 * 
 * When distributing Covered Code, include this CDDL
 * Header Notice in each file and include the License file
 * at https://woodstock.dev.java.net/public/CDDLv1.0.html.
 * If applicable, add the following below the CDDL Header,
 * with the fields enclosed by brackets [] replaced by
 * you own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Copyright 2008 Sun Microsystems, Inc. All rights reserved.
 */

woodstock4_3._dojo.provide("woodstock4_3.widget._jsfx.table2RowGroup");

woodstock4_3._dojo.require("woodstock4_3.json");
woodstock4_3._dojo.require("woodstock4_3.widget.common");
woodstock4_3._dojo.require("woodstock4_3.widget._jsfx.common");
woodstock4_3._dojo.require("woodstock4_3.widget._jsfx.dynaFaces");
woodstock4_3._dojo.require("woodstock4_3.widget.table2RowGroup");

/**
 * @class This class contains functions to obtain data asynchronously using JSF
 * Extensions as the underlying transfer protocol.
 * @static
 * @private
 */
woodstock4_3.widget._jsfx.table2RowGroup = {
    /**
     * This function is used to process scroll events with Object literals.
     *
     * @param props Key-Value pairs of properties.
     * @config {String} id The widget id.
     * @config {int} row The first row to be rendered.
     * @return {boolean} true if successful; otherwise, false.
     * @private
     */
    _processScrollEvent: function(props) {
        if (props == null) {
            return false;
        }

        // Dynamic Faces requires a DOM node as the source property.
        var domNode = document.getElementById(props.id);

        // Generate AJAX request using the JSF Extensions library.
        DynaFaces.fireAjaxTransaction(
            (domNode) ? domNode : document.forms[0], {
            execute: "none",
            render: props.id,
            replaceElement: woodstock4_3.widget._jsfx.table2RowGroup._scrollCallback,
            xjson: {
                id: props.id,
                event: "scroll",
                first: props.first
            }
        });
        return true;
    },

    /**
     * This function is used to process scroll events with Object literals.
     *
     * @param props Key-Value pairs of properties.
     * @config {String} id The widget id.
     * @config {int} row The first row to be rendered.
     * @return {boolean} true if successful; otherwise, false.
     * @private
     */
    _processSortEvent: function(props) {
        if (props == null) {
            return false;
        }

        // Dynamic Faces requires a DOM node as the source property.
        var domNode = document.getElementById(props.id);

        // Generate AJAX request using the JSF Extensions library.
        DynaFaces.fireAjaxTransaction(
            (domNode) ? domNode : document.forms[0], {
            execute: "none",
            render: props.id,
            replaceElement: woodstock4_3.widget._jsfx.table2RowGroup._sortCallback,
            xjson: {
                id: props.id,                
                event: "sort",
                colId: props.table2colId,
                sortOrder: props.sortOrder
            }
        });        
        return true;
    },

    /**
     * This function is used to update widgets.
     *
     * @param {String} id The widget id.
     * @param {String} content The content returned by the AJAX response.
     * @param {Object} closure The closure argument provided to the Ajax transaction.
     * @param {Object} xjson The xjson argument provided to the Ajax transaction.
     * @return {boolean} true if successful; otherwise, false.
     * @private
     */
    _scrollCallback: function(id, content, closure, xjson) {
        if (id == null || content == null) {
            return false;
        }

        // Parse JSON text.
        var props = woodstock4_3.json.parse(content);

        // Reject duplicate AJAX requests.
        var widget = woodstock4_3.widget.common.getWidget(id);
        if (widget.first != xjson.first) {
            return false;
        }        
        // Add rows.
        widget.addRows(props.rows);

        // Publish an event for custom AJAX implementations to listen for.
        woodstock4_3._dojo.publish(
            woodstock4_3.widget.table2RowGroup.event.scroll.endTopic, [props]);
        return true;
    },
    
    /**
     * This function is used to update widgets.
     *
     * @param {String} id The widget id.
     * @param {String} content The content returned by the AJAX response.
     * @param {Object} closure The closure argument provided to the Ajax transaction.
     * @param {Object} xjson The xjson argument provided to the Ajax transaction.
     * @return {boolean} true if successful; otherwise, false.
     * @private
     */
    _sortCallback: function(id, content, closure, xjson) {
        if (id == null || content == null) {
            return false;
        }

        // Parse JSON text.
        var props = woodstock4_3.json.parse(content);
        // Reject duplicate AJAX requests.
        var widget = woodstock4_3.widget.common.getWidget(id); 
        widget.setProps(props);
  
        // Publish an event for custom AJAX implementations to listen for.
        woodstock4_3._dojo.publish(
            woodstock4_3.widget.table2RowGroup.event.sort.endTopic, [props]);
        return true;
    }
};
    
// Listen for Widget events.
woodstock4_3._dojo.subscribe(woodstock4_3.widget.table2RowGroup.event.refresh.beginTopic,
    woodstock4_3.widget._jsfx.common, "_processRefreshEvent");
woodstock4_3._dojo.subscribe(woodstock4_3.widget.table2RowGroup.event.scroll.beginTopic,
    woodstock4_3.widget._jsfx.table2RowGroup, "_processScrollEvent");
woodstock4_3._dojo.subscribe(woodstock4_3.widget.table2RowGroup.event.pagination.next.beginTopic,
    woodstock4_3.widget._jsfx.table2RowGroup, "_processScrollEvent");
woodstock4_3._dojo.subscribe(woodstock4_3.widget.table2RowGroup.event.sort.beginTopic,
    woodstock4_3.widget._jsfx.table2RowGroup, "_processSortEvent");
